@@ -9,8 +9,9 @@
     :license: BSD, see LICENSE for more details.
 """
 from __future__ import absolute_import
-from bisect import bisect_left
+from bisect import bisect_right
 from collections import namedtuple
+import operator
 try:
     from weakref import WeakSet
 except ImportError:
@@ -19,12 +20,14 @@ except ImportError:
 from six.moves import map, zip
 from sortedcontainers import SortedList, SortedListWithKey
 
-from .common import ADD, REMOVE, TIME, VALUE, inf, now_or
+from .common import ADD, REMOVE, TIME, VALUE
+from .common import ERROR, OK, ONCE, CLAMP, inf, now_or  # to export
 from .deterministic import Determination, Segment
 
 
-__all__ = ['Gauge', 'Momentum', 'inf', 'now_or']
-__version__ = '0.3.0'
+__version__ = '0.3.1'
+__all__ = ['Gauge', 'Momentum',
+           'ERROR', 'OK', 'ONCE', 'CLAMP', 'inf', 'now_or']
 
 
 class Gauge(object):
@@ -50,20 +53,19 @@ class Gauge(object):
     #: The gauge to indicate minimum value.
     min_gauge = None
 
-    #: A weak set of gauges that refer the gauge as a limit gauge.
-    referring_gauges = None
-
     def __init__(self, value, max, min=0, at=None):
         self.__preinit__()
         at = now_or(at)
         self.base = (at, value)
-        self._set_limits(max, min, at=at, _incomplete=True)
+        self._set_range(max, min, at=at, _incomplete=True)
 
     def __preinit__(self):
         """Called by :meth:`__init__` and :meth:`__setstate__`."""
-        self.referring_gauges = WeakSet()
         self.momenta = SortedListWithKey(key=lambda m: m[2])  # sort by until
+        # the momentum events.
         self._events = SortedList()
+        # a weak set of gauges that refer the gauge as a limit gauge.
+        self._limited_gauges = WeakSet()
 
     @property
     def determination(self):
@@ -88,15 +90,14 @@ class Gauge(object):
         You don't need to call this method because all mutating methods such as
         :meth:`incr` or :meth:`add_momentum` calls it.
         """
-        # invalidate linked gauges together.  A linked gauge refers this gauge
-        # as a limit.
-        for gauge in self.referring_gauges:
-            gauge.invalidate()
         # remove the cached determination.
         try:
             del self._determination
         except AttributeError:
             pass
+        # invalidate limited gauges together.
+        for gauge in self._limited_gauges:
+            gauge._limit_gauge_invalidated(self)
 
     def get_max(self, at=None):
         """Predicts the current maximum value."""
@@ -118,13 +119,13 @@ class Gauge(object):
     #: The alias of :meth:`get_min`.
     min = get_min
 
-    def _set_limits(self, max_=None, min_=None, at=None, _incomplete=False):
+    def _set_range(self, max_=None, min_=None, at=None, _incomplete=False):
         at = now_or(at)
         forget_until = at
         # _incomplete=True when __init__() calls it.
         if not _incomplete:
             value = self.get(at)
-            inside_since = self.determination.inside_since
+            in_range_since = self.determination.in_range_since
         items = [('max', max_, self.max_gauge, min),
                  ('min', min_, self.min_gauge, max)]
         for name, limit, prev_limit_gauge, clamp in items:
@@ -132,7 +133,7 @@ class Gauge(object):
                 continue
             if prev_limit_gauge is not None:
                 # unlink from the previous limit gauge.
-                prev_limit_gauge.referring_gauges.discard(self)
+                prev_limit_gauge._limited_gauges.discard(self)
             if isinstance(limit, Gauge):
                 limit_gauge, limit_value = limit, limit.get(at)
                 forget_until = min(forget_until, limit_gauge.base[TIME])
@@ -146,14 +147,14 @@ class Gauge(object):
             else:
                 setattr(self, value_attr, None)
                 setattr(self, gauge_attr, limit_gauge)
-                limit_gauge.referring_gauges.add(self)
-            if _incomplete or inside_since is None:
+                limit_gauge._limited_gauges.add(self)
+            if _incomplete or in_range_since is None:
                 continue
-            elif inside_since <= at:
+            elif in_range_since <= at:
                 value = clamp(value, limit_value)
         if _incomplete:
             return
-        self.forget_past(value, at=forget_until)
+        return self.forget_past(value, at=forget_until)
 
     def set_max(self, max, at=None):
         """Changes the maximum.
@@ -161,7 +162,7 @@ class Gauge(object):
         :param max: a number or gauge to set as the maximum.
         :param at: the time to change.  (default: now)
         """
-        self._set_limits(max_=max, at=at)
+        return self._set_range(max_=max, at=at)
 
     def set_min(self, min, at=None):
         """Changes the minimum.
@@ -169,40 +170,29 @@ class Gauge(object):
         :param min: a number or gauge to set as the minimum.
         :param at: the time to change.  (default: now)
         """
-        self._set_limits(min_=min, at=at)
+        return self._set_range(min_=min, at=at)
 
-    def set_limits(self, max=None, min=None, at=None):
+    def set_range(self, max=None, min=None, at=None):
         """Changes the both of maximum and minimum at once.
 
         :param max: a number or gauge to set as the maximum.  (optional)
         :param min: a number or gauge to set as the minimum.  (optional)
         :param at: the time to change.  (default: now)
         """
-        return self._set_limits(max, min, at=at)
-
-    def clamp(self, value, at=None):
-        """Clamps by the limits at the given time.
-
-        :param value: the value to clamp.
-        :param at: the time to get limits.  (default: now)
-        """
-        at = now_or(at)
-        max_ = self.get_max(at)
-        if value > max_:
-            return max_
-        min_ = self.get_min(at)
-        if value < min_:
-            return min_
-        return value
+        return self._set_range(max, min, at=at)
 
     def _predict(self, at=None):
+        """Predicts the current value and velocity.
+
+        :param at: the time to observe.  (default: now)
+        """
         at = now_or(at)
         determination = self.determination
         if len(determination) == 1:
-            # skip bisect_left() because it is expensive
+            # skip bisect_right() because it is expensive
             x = 0
         else:
-            x = bisect_left(determination, (at,))
+            x = bisect_right(determination, (at, +inf))
         if x == 0:
             return (determination[0][VALUE], 0.)
         try:
@@ -212,10 +202,10 @@ class Gauge(object):
         time1, value1 = determination[x - 1]
         value = Segment._calc_value(at, time1, time2, value1, value2)
         velocity = Segment._calc_velocity(time1, time2, value1, value2)
-        if determination.inside_since is None:
+        if determination.in_range_since is None:
             pass
-        elif determination.inside_since <= time1:
-            value = self.clamp(value, at=at)
+        elif determination.in_range_since <= time1:
+            value = self._clamp(value, at=at)
         return (value, velocity)
 
     def get(self, at=None):
@@ -238,13 +228,13 @@ class Gauge(object):
         """Predicts the final value."""
         return self.determination[-1][VALUE]
 
-    def incr(self, delta, over=False, clamp=False, at=None):
+    def incr(self, delta, outbound=ERROR, at=None):
         """Increases the value by the given delta immediately.  The
         determination would be changed.
 
         :param delta: the value to increase.
-        :param over: allow out of the range.
-        :param clamp: clamp the value in the range.
+        :param outbound: the strategy to control modification to out of the
+                         range.  (default: ERROR)
         :param at: the time to increase.  (default: now)
 
         :raises ValueError: the value is out of the range.
@@ -252,53 +242,72 @@ class Gauge(object):
         at = now_or(at)
         prev_value = self.get(at=at)
         value = prev_value + delta
-        max_, min_ = self.get_max(at), self.get_min(at)
-        if over:
-            pass
-        elif clamp:
-            if delta > 0 and value > max_:
-                value = max(prev_value, max_)
-            elif delta < 0 and value < min_:
-                value = min(prev_value, min_)
-        else:
-            if delta > 0 and value > max_:
-                raise ValueError('The value to set is bigger than the '
-                                 'maximum ({0} > {1})'.format(value, max_))
-            elif delta < 0 and value < min_:
-                raise ValueError('The value to set is smaller than the '
-                                 'minimum ({0} < {1})'.format(value, min_))
-        self.forget_past(value, at=at)
-        return value
+        if outbound == ONCE:
+            outbound = OK if self.in_range(at) else ERROR
+        if outbound != OK:
+            items = [(
+                self.get_max, max, operator.gt,
+                'The value to set is bigger than the maximum ({0} > {1})'
+            ), (
+                self.get_min, min, operator.lt,
+                'The value to set is smaller than the minimum ({0} < {1})'
+            )]
+            for get_limit, clamp, cmp_, error_form in items:
+                if not cmp_(delta, 0):
+                    continue
+                limit = get_limit(at)
+                if not cmp_(value, limit):
+                    continue
+                if outbound == ERROR:
+                    raise ValueError(error_form.format(value, limit))
+                elif outbound == CLAMP:
+                    value = clamp(prev_value, limit)
+                    break
+        return self.forget_past(value, at=at)
 
-    def decr(self, delta, over=False, clamp=False, at=None):
+    def decr(self, delta, outbound=ERROR, at=None):
         """Decreases the value by the given delta immediately.  The
         determination would be changed.
 
         :param delta: the value to decrease.
-        :param over: allow out of the range.
-        :param clamp: clamp the value in the range.
+        :param outbound: the strategy to control modification to out of the
+                         range.  (default: ERROR)
         :param at: the time to decrease.  (default: now)
 
         :raises ValueError: the value is out of the range.
         """
-        return self.incr(-delta, over=over, clamp=clamp, at=at)
+        return self.incr(-delta, outbound=outbound, at=at)
 
-    def set(self, value, over=False, clamp=False, at=None):
+    def set(self, value, outbound=ERROR, at=None):
         """Sets the current value immediately.  The determination would be
         changed.
 
         :param value: the value to set.
-        :param over: allow out of the range.
-        :param clamp: clamp the value in the range.
+        :param outbound: the strategy to control modification to out of the
+                         range.  (default: ERROR)
         :param at: the time to set.  (default: now)
 
         :raises ValueError: the value is out of the range.
         """
         at = now_or(at)
-        if clamp and not over:
-            value = self.clamp(value, at=at)
         delta = value - self.get(at=at)
-        return self.incr(delta, over=over, clamp=clamp, at=at)
+        return self.incr(delta, outbound=outbound, at=at)
+
+    def _clamp(self, value, at=None):
+        at = now_or(at)
+        max_ = self.get_max(at)
+        if value > max_:
+            return max_
+        min_ = self.get_min(at)
+        if value < min_:
+            return min_
+        return value
+
+    def clamp(self, at=None):
+        """Clamps the current value."""
+        at = now_or(at)
+        value = self._clamp(self.get(at), at=at)
+        return self.set(value, outbound=OK, at=at)
 
     def when(self, value, after=0):
         """When the gauge reaches to the goal value.
@@ -333,6 +342,17 @@ class Gauge(object):
                 ratio = (value - value1) / float(value2 - value1)
                 yield (time1 + (time2 - time1) * ratio)
 
+    def in_range(self, at=None):
+        """Whether the gauge is between the range at the given time.
+
+        :param at: the time to check.  (default: now)
+        """
+        in_range_since = self.determination.in_range_since
+        if in_range_since is None:
+            return False
+        at = now_or(at)
+        return in_range_since <= at
+
     def _make_momentum(self, velocity_or_momentum, since=None, until=None):
         """Makes a :class:`Momentum` object by the given arguments.
 
@@ -401,37 +421,26 @@ class Gauge(object):
             self.momenta.remove(momentum)
         except ValueError:
             raise ValueError('{0} not in the gauge'.format(momentum))
+        since, until = momentum.since, momentum.until
+        self._events.remove((since, ADD, momentum))
+        if until != +inf:
+            self._events.remove((until, REMOVE, momentum))
         self.invalidate()
+        return momentum
 
     def momentum_events(self):
         """Yields momentum adding and removing events.  An event is a tuple of
         ``(time, ADD|REMOVE, momentum)``.
         """
         yield (self.base[TIME], None, None)
+        momentum_ids = set(id(m) for m in self.momenta)
         for time, method, momentum in list(self._events):
-            if momentum not in self.momenta:
+            if id(momentum) not in momentum_ids:
                 self._events.remove((time, method, momentum))
                 continue
             yield time, method, momentum
         yield (+inf, None, None)
 
-    def _limit_gauge_rebased(self, limit_gauge, limit_value, at=None):
-        """Be clamped by changed limit gauge."""
-        at = max(now_or(at), self.base[TIME])
-        value = self.get(at)
-        inside_since = self.determination.inside_since
-        if inside_since is None or at != self.base[TIME] and at < inside_since:
-            pass
-        else:
-            if limit_gauge is self.max_gauge:
-                clamp = min
-            elif limit_gauge is self.min_gauge:
-                clamp = max
-            else:
-                raise ValueError('The limit is neither max nor min')
-            value = clamp(value, limit_value)
-        self.forget_past(value, at=at)
-
     def _rebase(self, value=None, at=None, remove_momenta_before=None):
         """Sets the base and removes momenta between indexes of ``start`` and
         ``stop``.
@@ -445,7 +454,7 @@ class Gauge(object):
         at = now_or(at)
         if value is None:
             value = self.get(at=at)
-        for gauge in self.referring_gauges:
+        for gauge in self._limited_gauges:
             gauge._limit_gauge_rebased(self, value, at=at)
         self.base = (at, value)
         del self.momenta[:remove_momenta_before]
@@ -459,7 +468,7 @@ class Gauge(object):
         :param value: the value to set coercively.
         :param at: the time base.  (default: now)
         """
-        return self._rebase(value, at=at)
+        return self._rebase(value, at=at, remove_momenta_before=None)
 
     def forget_past(self, value=None, at=None):
         """Discards the momenta which doesn't effect anymore.
@@ -469,20 +478,43 @@ class Gauge(object):
         """
         at = now_or(at)
         x = self.momenta.bisect_left((-inf, -inf, at))
-        value = self._rebase(value, at=at, remove_momenta_before=x)
-        return value
+        return self._rebase(value, at=at, remove_momenta_before=x)
+
+    def _limit_gauge_invalidated(self, limit_gauge):
+        """The callback function which will be called at a limit gauge is
+        invalidated.
+        """
+        self.invalidate()
+
+    def _limit_gauge_rebased(self, limit_gauge, limit_value, at=None):
+        """The callback function which will be called at a limit gauge is
+        rebased.
+        """
+        at = max(now_or(at), self.base[TIME])
+        value = self.get(at)
+        if self.in_range(at):
+            if limit_gauge is self.max_gauge:
+                clamp = min
+            elif limit_gauge is self.min_gauge:
+                clamp = max
+            else:
+                raise ValueError('The limit is neither max nor min')
+            value = clamp(value, limit_value)
+        self.forget_past(value, at=at)
 
     def __getstate__(self):
         return (self.base, list(map(tuple, self.momenta)),
-                self.max_value, self.max_gauge,
-                self.min_value, self.min_gauge)
+                self.max_value, self.max_gauge, self.min_value, self.min_gauge)
 
     def __setstate__(self, state):
-        base, momenta, max_value, max_gauge, min_value, min_gauge = state
         self.__preinit__()
+        base, momenta, max_value, max_gauge, min_value, min_gauge = state
         self.base = base
         self.max_value, self.max_gauge = max_value, max_gauge
         self.min_value, self.min_gauge = min_value, min_gauge
+        for limit_gauge in [max_gauge, min_gauge]:
+            if limit_gauge is not None:
+                limit_gauge._limited_gauges.add(self)
         for momentum in momenta:
             self.add_momentum(*momentum)
 
