@@ -23,28 +23,29 @@ class Determination(list):
     :param determining: a :meth:`Gauge.determine` iterator.
     """
 
-    #: The time when the gauge starts to be inside of the limits.
-    inside_since = None
+    #: The time when the gauge starts to be in_range of the limits.
+    in_range_since = None
 
     @staticmethod
-    def walk_lines(gauge, src, is_gauge=False):
-        if is_gauge:
-            determination = src.determination
-            first, last = determination[0], determination[-1]
-            if gauge.base[TIME] < first[TIME]:
-                yield Horizon(gauge.base[TIME], first[TIME], first[VALUE])
-            zipped_determination = zip(determination[:-1], determination[1:])
-            for (time1, value1), (time2, value2) in zipped_determination:
-                yield Segment(time1, time2, value1, value2)
-            yield Horizon(last[TIME], +inf, last[VALUE])
-        else:
-            yield Horizon(gauge.base[TIME], +inf, src)
+    def value_lines(gauge, value):
+        yield Horizon(gauge.base[TIME], +inf, value)
 
-    def determine(self, time, value, inside=True):
+    @staticmethod
+    def gauge_lines(gauge, other_gauge):
+        determination = other_gauge.determination
+        first, last = determination[0], determination[-1]
+        if gauge.base[TIME] < first[TIME]:
+            yield Horizon(gauge.base[TIME], first[TIME], first[VALUE])
+        zipped_determination = zip(determination[:-1], determination[1:])
+        for (time1, value1), (time2, value2) in zipped_determination:
+            yield Segment(time1, time2, value1, value2)
+        yield Horizon(last[TIME], +inf, last[VALUE])
+
+    def determine(self, time, value, in_range=True):
         if self and self[-1][TIME] == time:
             return
-        if inside and self.inside_since is None:
-            self.inside_since = time
+        if in_range and self.in_range_since is None:
+            self.in_range_since = time
         self.append((time, value))
 
     def __init__(self, gauge):
@@ -55,14 +56,12 @@ class Determination(list):
         velocity, velocities = 0, []
         bound, overlapped = None, False
         # boundaries.
-        ceil_lines_iter = \
-            self.walk_lines(gauge, gauge.max_value) \
-            if gauge.max_gauge is None else \
-            self.walk_lines(gauge, gauge.max_gauge, is_gauge=True)
-        floor_lines_iter = \
-            self.walk_lines(gauge, gauge.min_value) \
-            if gauge.min_gauge is None else \
-            self.walk_lines(gauge, gauge.min_gauge, is_gauge=True)
+        ceil_lines_iter = (self.value_lines(gauge, gauge.max_value)
+                           if gauge.max_gauge is None else
+                           self.gauge_lines(gauge, gauge.max_gauge))
+        floor_lines_iter = (self.value_lines(gauge, gauge.min_value)
+                            if gauge.min_gauge is None else
+                            self.gauge_lines(gauge, gauge.min_gauge))
         ceil = Boundary(ceil_lines_iter, operator.lt)
         floor = Boundary(floor_lines_iter, operator.gt)
         boundaries = [ceil, floor]
@@ -119,6 +118,7 @@ class Determination(list):
                 for boundary in walked_boundaries:
                     # find the intersection with a boundary.
                     try:
+                        # intersection = boundary.line.intersect(line)
                         intersection = line.intersect(boundary.line)
                     except ValueError:
                         continue
@@ -150,7 +150,7 @@ class Determination(list):
                 break
             # determine the final node in the current itreration.
             value += velocity * (until - since)
-            self.determine(until, value, inside=(bound is None or overlapped))
+            self.determine(until, value, in_range=bound is None or overlapped)
             # prepare the next iteration.
             if method == ADD:
                 velocities.append(momentum.velocity)
@@ -227,25 +227,32 @@ class Line(object):
 
         :raises ValueError: there's no intersection.
         """
-        intercept_delta = line.intercept() - self.intercept()
-        velocity_delta = self.velocity - line.velocity
+        lines = [self, line]
+        lines.sort(key=intersection_reliability, reverse=True)
+        left, right = lines  # right is more reliable.
+        intercept_delta = right.intercept() - left.intercept()
+        velocity_delta = left.velocity - right.velocity
         try:
             time = intercept_delta / velocity_delta
         except ZeroDivisionError:
             raise ValueError('Parallel line given')
-        since = max(self.since, line.since)
-        until = min(self.until, line.until)
+        since = max(left.since, right.since)
+        until = min(left.until, right.until)
         if since <= time <= until:
             pass
         else:
             raise ValueError('Intersection not in the time range')
-        value = self.get(time)
+        value = left.get(time)
         return (time, value)
 
     def intercept(self):
         """Gets the value-intercept. (Y-intercept)"""
         return self.value - self.velocity * self.since
 
+    def __repr__(self, string=''):
+        return ('<{0}{1} for {2!r}~{3!r}>'
+                ''.format(type(self).__name__, string, self.since, self.until))
+
 
 class Horizon(Line):
     """A line which has no velocity."""
@@ -261,6 +268,9 @@ class Horizon(Line):
     def _later(self, at):
         return self.value
 
+    def __repr__(self):
+        return super(Horizon, self).__repr__(' {0:.2f}'.format(self.value))
+
 
 class Ray(Line):
     """A line based on starting value and velocity."""
@@ -280,6 +290,10 @@ class Ray(Line):
     def _later(self, at):
         return self._get(self.until)
 
+    def __repr__(self):
+        string = ' {0:.2f}{1:+.2f}/s'.format(self.value, self.velocity)
+        return super(Ray, self).__repr__(string)
+
 
 class Segment(Line):
     """A line based on starting and ending value."""
@@ -319,6 +333,17 @@ class Segment(Line):
     def _later(self, at):
         return self.final
 
+    def __repr__(self):
+        string = ' {0:.2f}~{1:.2f}'.format(self.value, self.final)
+        return super(Segment, self).__repr__(string)
+
+
+#: The reliability map of line classes for precise intersection.
+_intersection_reliabilities = {Horizon: 3, Ray: 2, Segment: 1}
+
+#: Sorting key to sort by intersection reliability.
+intersection_reliability = lambda l: _intersection_reliabilities[type(l)]
+
 
 class Boundary(object):
 
